//
// Open Service Platform
// Copyright (c) 2012 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

/**
 * @file	FIo_ChannelService.h
 * @brief	This is the header file for the _ChannelService class.
 *
 * This file contains the declarations of _ChannelService.
 */

#ifndef _FIO_INTERNAL_CHANNEL_SERVICE_H_
#define _FIO_INTERNAL_CHANNEL_SERVICE_H_

#include <vector>

#include <FBaseResult.h>
#include <FBaseObject.h>
#include <FBaseColIList.h>
#include <FBaseColHashMapT.h>
#include <FOspConfig.h>
#include "FIo_IChannelService.h"
#include "FIo_IChannelServiceEventListener.h"

namespace IPC
{
class Message;
}

namespace Tizen { namespace Io
{
class _IChannelServiceStub;
class _ChannelCAppStub;

enum _ChannelType
{
	OSPAPP_CHANNEL = 0,
	CAPP_CHANNEL,
	WEBAPP_CHANNEL
};

class _OSP_EXPORT_ _ChannelService
	: public Tizen::Base::Object
	, public Tizen::Io::_IChannelService
{
public:
	_ChannelService(void);

	virtual ~_ChannelService(void);

	virtual result Construct(_IChannelServiceStub& stub);

	// _IChannelService
	virtual result RegisterChannel(const Tizen::Base::String& channelId, const _IChannelServiceEventListener& listener);
	virtual result UnregisterChannel(const Tizen::Base::String& channelId);

	virtual result SendRequest(const Tizen::Base::String& src, const Tizen::Base::String& dest, const Tizen::Base::Collection::ArrayList& args, int messageId);
	virtual result SendNullRequest(const Tizen::Base::String& src, const Tizen::Base::String& dest, int messageId);

	virtual result SendResponse(const Tizen::Base::String& src, const Tizen::Base::String& dest, const Tizen::Base::Collection::ArrayList& args, int messageId);
	virtual result SendNullResponse(const Tizen::Base::String& src, const Tizen::Base::String& dest, int messageId);


	bool IsChannelRegistered(const Tizen::Base::String& channelId);

	result RegisterChannel(const Tizen::Base::String& channelId, int clientId, unsigned int type = OSPAPP_CHANNEL);

	result RegisterChannel(const Tizen::Base::String& channelId, int clientId, void* pGIOChannel);

	result UnregisterChannel(int clientId);

private:

	struct _ChannelInfo
	{
		Tizen::Base::String channelId;
		int clientId;
		void* pGIOChannel;
		unsigned int type;
	};

	class _StringHashProvider
		: public Tizen::Base::Collection::IHashCodeProviderT<Tizen::Base::String>
	{
	public:
		virtual int GetHashCode(const Tizen::Base::String& value) const
		{
			return value.GetHashCode();
		}
	};

	class _StringComparer
		: public Tizen::Base::Collection::IComparerT<Tizen::Base::String>
	{
	public:
		virtual result Compare(const Tizen::Base::String& string1, const Tizen::Base::String& string2, int& cmp) const
		{
			cmp = string1.CompareTo(string2);
			return E_SUCCESS;
		}
	};
private:
	_IChannelServiceStub* __pIChannelServiceStub;
	Tizen::Base::Collection::HashMapT <Tizen::Base::String, _ChannelInfo*> __channels;

}; // _ChannelService


} } // Tizen::Io

#endif // _FIO_INTERNAL_CHANNEL_SERVICE_STUB_H_
